==> unicore-gateway-6.4.0-2.src.tar.gz: src/main/java/eu/unicore/gateway/base/CustomSslSocketConnector.java <==
/*********************************************************************************
 * Copyright (c) 2008 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
package eu.unicore.gateway.base;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.CertificateException;

import javax.net.ssl.HandshakeCompletedEvent;
import javax.net.ssl.HandshakeCompletedListener;
import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;

import org.apache.log4j.Logger;
import org.mortbay.io.EndPoint;
import org.mortbay.jetty.HttpConnection;
import org.mortbay.jetty.security.Password;
import org.mortbay.jetty.security.SslSocketConnector;
import org.mortbay.resource.Resource;

import eu.unicore.gateway.properties.GatewayProperties;
import eu.unicore.gateway.util.CustomHttpConnection;
import eu.unicore.gateway.util.LogUtil;
import eu.unicore.gateway.util.TruststoreUtil;


/**
 * Slight extension to the Jetty SslSocketConnector, allowing to customise trust
 * management. Will also log the address of the remote host trying to 
 * establish a connection.
 */
public class CustomSslSocketConnector extends SslSocketConnector
{

	private final static Logger log=LogUtil.getLogger(LogUtil.CONNECTIONS,CustomSslSocketConnector.class);

	private transient Password password;
	private transient Password keyPassword;
	private transient Password trustPassword;
	private GatewayProperties gwProps;

	public CustomSslSocketConnector(GatewayProperties gwProps)
	{
		super();
		this.gwProps = gwProps;
	}

	@Override
	protected void configure(Socket socket)throws IOException{
		InetSocketAddress peer=(InetSocketAddress)socket.getRemoteSocketAddress();
		if(log.isDebugEnabled() && peer!=null && peer.getAddress()!=null){
			log.debug("Connection attempt from "+peer.getAddress().getHostAddress());
			if(socket instanceof SSLSocket){
				SSLSocket ssl=(SSLSocket)socket;
				ssl.addHandshakeCompletedListener(new HandshakeCompletedListener(){

					public void handshakeCompleted(HandshakeCompletedEvent hce) {
						String msg;
						try{
							msg=String.valueOf(hce.getPeerPrincipal());
						}catch(SSLPeerUnverifiedException spe){
							msg="Problem: Unverified peer!";
						}
						log.debug("SSL connection to <"+msg+"> established.");
					}

				});
			}
		}
		super.configure(socket);
	}

	@Override
	protected HttpConnection newHttpConnection(EndPoint endpoint){
		return new CustomHttpConnection(this, endpoint, getServer(), gwProps);
	}

	public void setPassword(String password){
		this.password=Password.getPassword(PASSWORD_PROPERTY,password,null);
		super.setPassword(password);
	}

	public void setTrustPassword(String password){
		trustPassword=Password.getPassword(PASSWORD_PROPERTY,password,null);
		super.setTrustPassword(password);
	}

	public void setKeyPassword(String password){
		keyPassword=Password.getPassword(KEYPASSWORD_PROPERTY,password,null);
		super.setKeyPassword(password);
	}

	protected SSLServerSocketFactory createFactory() throws Exception  {
		if (password==null)
			password=new Password("");
		if (keyPassword==null)
			keyPassword=password;
		if (trustPassword==null)
			trustPassword=password;


		if (getTruststore()==null)
		{
			setTruststore(getKeystore());
			setTruststoreType(getKeystoreType());
		}
		KeyManager[] keyManagers = null;
		if (getKeystore()!= null)
		{
			KeyStore keyStore = KeyStore.getInstance(getKeystoreType());
			if (password == null) 
				throw new SSLException("password is not set");
			keyStore.load(Resource.newResource(getKeystore()).getInputStream(), password.toString().toCharArray());

			KeyManagerFactory keyManagerFactory = KeyManagerFactory.getInstance(getSslKeyManagerFactoryAlgorithm());        
			if (keyPassword == null) 
				throw new SSLException("keypassword is not set");
			keyManagerFactory.init(keyStore,keyPassword.toString().toCharArray());
			keyManagers = keyManagerFactory.getKeyManagers();
		}

		TrustManager[] trustManagers = getTrustManagers();

		SecureRandom secureRandom = getSecureRandomAlgorithm()==null?null:SecureRandom.getInstance(getSecureRandomAlgorithm());

		SSLContext context = getProvider()==null?SSLContext.getInstance(getProtocol()):SSLContext.getInstance(getProtocol(), getProvider());

		context.init(keyManagers, trustManagers, secureRandom);

		return context.getServerSocketFactory();
	}


	protected TrustManager[] getTrustManagers()throws IOException,NoSuchAlgorithmException,CertificateException,KeyStoreException{
		TrustManager[] trustManagers = null;
		if (getTruststore()!= null)
		{
			KeyStore trustStore =loadTruststore();
			TrustManagerFactory trustManagerFactory = TrustManagerFactory.getInstance(getSslTrustManagerFactoryAlgorithm());
			trustManagerFactory.init(trustStore);
			trustManagers = trustManagerFactory.getTrustManagers();
		}
		TrustManager[] res=new X509TrustManager[trustManagers.length];
		for(int i=0;i<res.length;i++){
			res[i]=decorate(trustManagers[i]);
		}
		return res;
	}

	protected TrustManager decorate(TrustManager tm){
		return tm instanceof X509TrustManager ? 
				new AuthSSLX509TrustManager((X509TrustManager)tm, gwProps): tm ;
	}

	protected KeyStore loadTruststore()throws IOException,NoSuchAlgorithmException,CertificateException,KeyStoreException {
		KeyStore trustStore=null;
		if("file".equalsIgnoreCase(getTruststoreType())){
			return TruststoreUtil.loadTruststoreFromPemFile(getTruststore());
		}
		else if(getTruststoreType().startsWith("dir")){
			return TruststoreUtil.loadTruststoreFromDirectory(getTruststore());
		}
		else{	
			trustStore= KeyStore.getInstance(getTruststoreType());
			trustStore.load(Resource.newResource(getTruststore()).getInputStream(), trustPassword.toString().toCharArray());
		}
		return trustStore;
	}

	

	
}

==> unicore-gateway6-4.2.0-0.src.tar.gz: src/main/java/eu/unicore/gateway/base/CustomSslSocketConnector.java <==
/*********************************************************************************
 * Copyright (c) 2008 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
package eu.unicore.gateway.base;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.CertificateException;

import javax.net.ssl.HandshakeCompletedEvent;
import javax.net.ssl.HandshakeCompletedListener;
import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;

import org.apache.log4j.Logger;
import org.mortbay.io.EndPoint;
import org.mortbay.jetty.HttpConnection;
import org.mortbay.jetty.security.Password;
import org.mortbay.jetty.security.SslSocketConnector;
import org.mortbay.resource.Resource;

import eu.unicore.gateway.properties.GatewayProperties;
import eu.unicore.gateway.util.CustomHttpConnection;
import eu.unicore.gateway.util.LogUtil;
import eu.unicore.security.util.KeystoreUtil;


/**
 * Slight extension to the Jetty SslSocketConnector, allowing to customise trust
 * management. Will also log the address of the remote host trying to 
 * establish a connection.
 */
public class CustomSslSocketConnector extends SslSocketConnector
{

	private final static Logger log=LogUtil.getLogger(LogUtil.CONNECTIONS,CustomSslSocketConnector.class);

	private Password password;
	private Password keyPassword;
	private Password trustPassword;
	private final GatewayProperties gwProps;

	public CustomSslSocketConnector(GatewayProperties gwProps)
	{
		super();
		this.gwProps = gwProps;
		setTruststoreType(null);
		setKeystoreType(null);
	}

	@Override
	protected void configure(Socket socket)throws IOException{
		InetSocketAddress peer=(InetSocketAddress)socket.getRemoteSocketAddress();
		if(log.isDebugEnabled() && peer!=null && peer.getAddress()!=null){
			log.debug("Connection attempt from "+peer.getAddress().getHostAddress());
			if(socket instanceof SSLSocket){
				SSLSocket ssl=(SSLSocket)socket;
				ssl.addHandshakeCompletedListener(new HandshakeCompletedListener(){

					public void handshakeCompleted(HandshakeCompletedEvent hce) {
						String msg;
						try{
							msg=String.valueOf(hce.getPeerPrincipal());
						}catch(SSLPeerUnverifiedException spe){
							msg="Problem: Unverified peer!";
						}
						log.debug("SSL connection to <"+msg+"> established.");
					}

				});
			}
		}
		super.configure(socket);
	}

	@Override
	protected HttpConnection newHttpConnection(EndPoint endpoint){
		return new CustomHttpConnection(this, endpoint, getServer(), gwProps);
	}

	public void setPassword(String password){
		this.password=Password.getPassword(PASSWORD_PROPERTY,password,null);
		super.setPassword(password);
	}

	public void setTrustPassword(String password){
		trustPassword=Password.getPassword(PASSWORD_PROPERTY,password,null);
		super.setTrustPassword(password);
	}

	public void setKeyPassword(String password){
		keyPassword=Password.getPassword(KEYPASSWORD_PROPERTY,password,null);
		super.setKeyPassword(password);
	}

	protected SSLServerSocketFactory createFactory() throws Exception  {
		if (password==null)
			password=new Password("");
		if (keyPassword==null)
			keyPassword=password;
		if (trustPassword==null)
			trustPassword=password;


		if (getTruststore()==null)
		{
			setTruststore(getKeystore());
			setTruststoreType(getKeystoreType());
		}
		KeyManager[] keyManagers = null;
		if (getKeystore()!= null)
		{
		        KeyStore keyStore = KeystoreUtil.loadKeyStore(getKeystore(), password.toString(), getKeystoreType());
			if (password == null) 
				throw new SSLException("password is not set");
			keyStore.load(Resource.newResource(getKeystore()).getInputStream(), password.toString().toCharArray());

			KeyManagerFactory keyManagerFactory = KeyManagerFactory.getInstance(getSslKeyManagerFactoryAlgorithm());        
			if (keyPassword == null) 
				throw new SSLException("keypassword is not set");
			keyManagerFactory.init(keyStore,keyPassword.toString().toCharArray());
			keyManagers = keyManagerFactory.getKeyManagers();
		}

		TrustManager[] trustManagers = getTrustManagers();

		SecureRandom secureRandom = getSecureRandomAlgorithm()==null?null:SecureRandom.getInstance(getSecureRandomAlgorithm());

		SSLContext context = getProvider()==null?SSLContext.getInstance(getProtocol()):SSLContext.getInstance(getProtocol(), getProvider());

		context.init(keyManagers, trustManagers, secureRandom);

		return context.getServerSocketFactory();
	}


	protected TrustManager[] getTrustManagers()throws IOException,NoSuchAlgorithmException,CertificateException,KeyStoreException{
		TrustManager[] trustManagers = null;
		if (getTruststore()!= null)
		{
			KeyStore trustStore =loadTruststore();
			TrustManagerFactory trustManagerFactory = TrustManagerFactory.getInstance(getSslTrustManagerFactoryAlgorithm());
			trustManagerFactory.init(trustStore);
			trustManagers = trustManagerFactory.getTrustManagers();
		}
		else{
			return new X509TrustManager[0];
		}
		TrustManager[] res=new X509TrustManager[trustManagers.length];
		for(int i=0;i<res.length;i++){
			res[i]=decorate(trustManagers[i]);
		}
		return res;
	}

	protected TrustManager decorate(TrustManager tm){
		return tm instanceof X509TrustManager ? 
				new AuthSSLX509TrustManager((X509TrustManager)tm, gwProps): tm ;
	}

	protected KeyStore loadTruststore()throws IOException,NoSuchAlgorithmException,CertificateException,KeyStoreException {
		String name=getTruststore();
		String passwd=trustPassword!=null?trustPassword.toString():"unknown";
		String type=getTruststoreType();
		return KeystoreUtil.loadTruststore(name, passwd, type);
	}

	
}
